#include "utf8_encode.h"
#include "xmalloc.h"

#include <iconv.h>
#include <string.h>
#include <errno.h>

int utf8_encode(const char *inbuf, const char *encoding, char **outbuf)
{
	const char *in;
	char *out;
	size_t inbuf_size, outbuf_size, i;
	iconv_t cd;
	int rc;

	cd = iconv_open("UTF-8", encoding);
	if (cd == (iconv_t)-1)
		return -1;
	inbuf_size = strlen(inbuf);
	outbuf_size = inbuf_size;
	for (i = 0; i < inbuf_size; i++) {
		unsigned char ch;
		
		ch = inbuf[i];
		if (ch > 127)
			outbuf_size++;
	}
	*outbuf = xnew(char, outbuf_size + 1);
	in = inbuf;
	out = *outbuf;
/* 	printf("%d %d\n", inbuf_size, outbuf_size); */
	rc = iconv(cd, (char **)&in, &inbuf_size, &out, &outbuf_size);
	*out = 0;
/* 	printf("%d %d\n", inbuf_size, outbuf_size); */
	if (rc == -1) {
		int save = errno;
		iconv_close(cd);
		free(*outbuf);
		*outbuf = NULL;
		errno = save;
		return -1;
	}
	rc = iconv_close(cd);
	if (rc == -1) {
		int save = errno;
		free(*outbuf);
		*outbuf = NULL;
		errno = save;
		return -1;
	}
/* 	printf("'%s'\n'%s'\n", inbuf, *outbuf); */
	return 0;
}
